using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;

using CMS.GlobalHelper;
using CMS.ExtendedControls;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.TreeEngine;

public partial class CMSModules_Content_FormControls_Documents_SelectSinglePath : CMS.FormControls.FormEngineUserControl
{
    #region "Variables"

    private int mSiteId = 0;
    private DialogConfiguration mConfig = null;
    private TreeProvider mTreeProvider = null;
    private bool mValueSetFailed = false;
    private string mValue = null;

    #endregion


    #region "Private properties"

    /// <summary>
    /// Returns TreeProvider object.
    /// </summary>
    private TreeProvider TreeProvider
    {
        get
        {
            if (this.mTreeProvider == null)
            {
                mTreeProvider = new TreeProvider(CMSContext.CurrentUser);
            }
            return this.mTreeProvider;
        }
    }


    /// <summary>
    /// Gets the configuration for Copy and Move dialog.
    /// </summary>
    private DialogConfiguration Config
    {
        get
        {
            if (this.mConfig == null)
            {
                this.mConfig = new DialogConfiguration();
                this.mConfig.HideLibraries = true;
                this.mConfig.ContentSelectedSite = CMSContext.CurrentSiteName;
                this.mConfig.HideAnchor = true;
                this.mConfig.HideAttachments = true;
                this.mConfig.HideContent = false;
                this.mConfig.HideEmail = true;
                this.mConfig.HideLibraries = true;
                this.mConfig.HideWeb = true;
                this.mConfig.EditorClientID = this.txtNodeId.ClientID;
                this.mConfig.OutputFormat = OutputFormatEnum.Custom;
                this.mConfig.CustomFormatCode = "selectpath";
                this.mConfig.SelectableContent = SelectableContentEnum.AllContent;
            }
            return this.mConfig;
        }
    }

    #endregion


    #region "Public properties"

    /// <summary>
    /// Gets or sets the enabled state of the control
    /// </summary>
    public override bool Enabled
    {
        get
        {
            return base.Enabled;
        }
        set
        {
            base.Enabled = value;
            this.txtPath.Enabled = value;
            this.btnSelectPath.Enabled = value;
        }
    }


    /// <summary>
    /// Gets or sets field value.
    /// </summary>
    public override object Value
    {
        get
        {
            return this.txtPath.Text;
        }
        set
        {
            try
            {
                this.txtPath.Text = (string)value;
            }
            catch
            {
                mValueSetFailed = true;
                mValue = (string)value;
            }
        }
    }


    /// <summary>
    /// Gets ClientID of the textbox with path
    /// </summary>
    public override string ValueElementID
    {
        get
        {
            return this.txtPath.ClientID;
        }
    }


    /// <summary>
    /// Gets or sets the ID of the site from which the path is selected.
    /// </summary>
    public int SiteID
    {
        get
        {
            return this.mSiteId;
        }
        set
        {
            this.mSiteId = value;
            if (value > 0)
            {
                this.Config.ContentSites = AvailableSitesEnum.OnlySingleSite;
                SiteInfo si = SiteInfoProvider.GetSiteInfo(value);
                if (si != null)
                {
                    this.Config.ContentSelectedSite = si.SiteName;
                }
            }
            else
            {
                this.Config.ContentSites = AvailableSitesEnum.All;
            }
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        if (mValueSetFailed)
        {
            if (!String.IsNullOrEmpty(Request.Form[this.txtPath.UniqueID]))
            {
                txtPath.Text = Request.Form[this.txtPath.UniqueID];
            }
            else
            {
                txtPath.Text = mValue;
            }
        }

        ScriptHelper.RegisterClientScriptBlock(this, typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);

        this.btnSelectPath.Text = ResHelper.GetString("general.select");
        this.btnSelectPath.OnClientClick = "modalDialog('" + GetDialogUrl() + "','PathSelection', '90%', '85%'); return false;";
        this.txtNodeId.TextChanged += new EventHandler(txtNodeId_TextChanged);
    }


    void txtNodeId_TextChanged(object sender, EventArgs e)
    {
        int nodeId = ValidationHelper.GetInteger(txtNodeId.Text, 0);
        if (nodeId > 0)
        {
            TreeNode node = this.TreeProvider.SelectSingleNode(nodeId);
            if (node != null)
            {
                SiteID = node.NodeSiteID;
                txtPath.Text = node.NodeAliasPath;
            }
        }
    }


    /// <summary>
    /// Returns Correct URL of the copy or move dialog.
    /// </summary>
    private string GetDialogUrl()
    {
        string url = CMSDialogHelper.GetDialogUrl(this.Config, this.IsLiveSite, false);
        string fullDomain = UrlHelper.GetFullDomain();

        // Remove first occurrence of domain
        int index = url.IndexOf(fullDomain);
        url = (index < 0) ? url : url.Remove(index, fullDomain.Length);

        url = url.Replace("https://", "").Replace("http://", "");
        url = UrlHelper.RemoveParameterFromUrl(url, "hash");
        url = UrlHelper.AddParameterToUrl(url, "selectionmode", "single");
        url = UrlHelper.AddParameterToUrl(url, "hash", QueryHelper.GetHash(url));
        url = UrlHelper.EncodeQueryString(url);
        return url;
    }
}
